/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */
#ifndef __GdomDocument_H__
#define __GdomDocument_H__
#include <webkit/webkitdefines.h>
#include "gdom/GdomAttr.h"
#include "gdom/GdomCDATASection.h"
#include "gdom/GdomCSSStyleDeclaration.h"
#include "gdom/GdomCanvasRenderingContext2D.h"
#include "gdom/GdomComment.h"
#include "gdom/GdomDOMSelection.h"
#include "gdom/GdomDocumentFragment.h"
#include "gdom/GdomElement.h"
#include "gdom/GdomEntityReference.h"
#include "gdom/GdomEvent.h"
#include "gdom/GdomEventTargetNode.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomNodeFilter.h"
#include "gdom/GdomNodeIterator.h"
#include "gdom/GdomNodeList.h"
#include "gdom/GdomProcessingInstruction.h"
#include "gdom/GdomRange.h"
#include "gdom/GdomText.h"
#include "gdom/GdomTreeWalker.h"
#include "gdom/GdomXPathExpression.h"
#include "gdom/GdomXPathNSResolver.h"
#include "gdom/GdomXPathResult.h"
#include "glib-object.h"


G_BEGIN_DECLS

typedef struct _GdomDocument GdomDocument;
typedef struct _GdomDocumentClass GdomDocumentClass;


#define GDOM_TYPE_DOCUMENT            (gdom_document_get_type())
#define GDOM_DOCUMENT(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GDOM_TYPE_DOCUMENT, GdomDocument))
#define GDOM_DOCUMENT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),  GDOM_TYPE_DOCUMENT, GdomDocument)Class)
#define GDOM_IS_DOCUMENT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GDOM_TYPE_DOCUMENT))
#define GDOM_IS_DOCUMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),  GDOM_TYPE_DOCUMENT))
#define GDOM_DOCUMENT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj),  GDOM_TYPE_DOCUMENT, GdomDocumentClass))

typedef struct _GdomDocumentPrivate GdomDocumentPrivate;

struct _GdomDocument {
    GdomEventTargetNode parent_instance;
    GdomDocumentPrivate *priv;
};

struct _GdomDocumentClass {
    GObjectClass parent_class;
};

WEBKIT_API GType
gdom_document_get_type (void);

WEBKIT_API GdomElement *
gdom_document_create_element (GdomDocument *thiz, gchar * tag_name);

WEBKIT_API GdomDocumentFragment *
gdom_document_create_document_fragment (GdomDocument *thiz);

WEBKIT_API GdomText *
gdom_document_create_text_node (GdomDocument *thiz, gchar * data);

WEBKIT_API GdomComment *
gdom_document_create_comment (GdomDocument *thiz, gchar * data);

WEBKIT_API GdomCDATASection *
gdom_document_create_cdata_section (GdomDocument *thiz, gchar * data);

WEBKIT_API GdomProcessingInstruction *
gdom_document_create_processing_instruction (GdomDocument *thiz, gchar * target, gchar * data);

WEBKIT_API GdomAttr *
gdom_document_create_attribute (GdomDocument *thiz, gchar * name);

WEBKIT_API GdomEntityReference *
gdom_document_create_entity_reference (GdomDocument *thiz, gchar * name);

WEBKIT_API GdomNodeList *
gdom_document_get_elements_by_tag_name (GdomDocument *thiz, gchar * tagname);

WEBKIT_API GdomNode *
gdom_document_import_node (GdomDocument *thiz, GdomNode * imported_node, gboolean deep);

WEBKIT_API GdomElement *
gdom_document_create_element_ns (GdomDocument *thiz, gchar * namespace_uri, gchar * qualified_name);

WEBKIT_API GdomAttr *
gdom_document_create_attribute_ns (GdomDocument *thiz, gchar * namespace_uri, gchar * qualified_name);

WEBKIT_API GdomNodeList *
gdom_document_get_elements_by_tag_name_ns (GdomDocument *thiz, gchar * namespace_uri, gchar * local_name);

WEBKIT_API GdomElement *
gdom_document_get_element_by_id (GdomDocument *thiz, gchar * element_id);

WEBKIT_API GdomNode *
gdom_document_adopt_node (GdomDocument *thiz, GdomNode * source);

WEBKIT_API GdomEvent *
gdom_document_create_event (GdomDocument *thiz, gchar * event_type);

WEBKIT_API GdomRange *
gdom_document_create_range (GdomDocument *thiz);

WEBKIT_API GdomNodeIterator *
gdom_document_create_node_iterator (GdomDocument *thiz, GdomNode * root, gulong what_to_show, GdomNodeFilter * filter, gboolean entity_reference_expansion);

WEBKIT_API GdomTreeWalker *
gdom_document_create_tree_walker (GdomDocument *thiz, GdomNode * root, gulong what_to_show, GdomNodeFilter * filter, gboolean entity_reference_expansion);

WEBKIT_API GdomCSSStyleDeclaration *
gdom_document_get_override_style (GdomDocument *thiz, GdomElement * element, gchar * pseudo_element);

WEBKIT_API GdomXPathExpression *
gdom_document_create_expression (GdomDocument *thiz, gchar * expression, GdomXPathNSResolver * resolver);

WEBKIT_API GdomXPathNSResolver *
gdom_document_create_ns_resolver (GdomDocument *thiz, GdomNode * node_resolver);

WEBKIT_API GdomXPathResult *
gdom_document_evaluate (GdomDocument *thiz, gchar * expression, GdomNode * context_node, GdomXPathNSResolver * resolver, gushort type, GdomXPathResult * in_result);

WEBKIT_API gboolean
gdom_document_exec_command (GdomDocument *thiz, gchar * command, gboolean user_interface, gchar * value);

WEBKIT_API gboolean
gdom_document_query_command_enabled (GdomDocument *thiz, gchar * command);

WEBKIT_API gboolean
gdom_document_query_command_indeterm (GdomDocument *thiz, gchar * command);

WEBKIT_API gboolean
gdom_document_query_command_state (GdomDocument *thiz, gchar * command);

WEBKIT_API gboolean
gdom_document_query_command_supported (GdomDocument *thiz, gchar * command);

WEBKIT_API gchar *
gdom_document_query_command_value (GdomDocument *thiz, gchar * command);

WEBKIT_API GdomNodeList *
gdom_document_get_elements_by_name (GdomDocument *thiz, gchar * element_name);

WEBKIT_API GdomElement *
gdom_document_element_from_point (GdomDocument *thiz, glong x, glong y);

WEBKIT_API GdomDOMSelection *
gdom_document_get_selection (GdomDocument *thiz);

WEBKIT_API GdomCSSStyleDeclaration *
gdom_document_create_css_style_declaration (GdomDocument *thiz);

WEBKIT_API GdomNodeList *
gdom_document_get_elements_by_class_name (GdomDocument *thiz, gchar * tagname);


/* XXX TODO: custom function gdom_document_query_selector */


/* XXX TODO: custom function gdom_document_query_selector_all */

G_END_DECLS

#endif /* __GdomDocument_H__ */
